#!/bin/sh

set -e

. ./common.sh

debug=0

if [ ${debug} -ne 0 ]
then
	IPTABLES="echo ${IPTABLES}"
	IP6TABLES="echo ${IP6TABLES}"
	IPSET="echo ${IPSET}"
fi

if [ $# -lt 1 ]
then
	echo "Usage: $(basename "$0") external_interface" 1>&2
	exit 64
fi

EXT_IF="$1"
if ! ip link show "${EXT_IF}" >/dev/null 2>&1
then
	echo "'${EXT_IF}' does not seem to be a valid interface"
	exit 1
fi

$IPTABLES -F
$IPTABLES -F INPUT
$IPTABLES -X

$IP6TABLES -F
$IP6TABLES -F INPUT
$IP6TABLES -X

# default policies
$IPTABLES -P INPUT DROP
$IPTABLES -P OUTPUT ACCEPT

$IP6TABLES -P INPUT DROP
$IP6TABLES -P OUTPUT ACCEPT

# accept local traffic
$IPTABLES -A INPUT -i lo -j ACCEPT

$IP6TABLES -A INPUT -i lo -j ACCEPT

# accept ICMP
$IPTABLES -A INPUT -p icmp -j ACCEPT

$IP6TABLES -A INPUT -p ipv6-icmp -j ACCEPT

# drop source-route rh0 headery things
$IP6TABLES -A INPUT -m rt --rt-type 0 -j DROP || echo "MISSING RT MATCH" 1>&2

# accept things we set up
$IPTABLES -A INPUT -m conntrack --ctstate RELATED,ESTABLISHED -j ACCEPT

$IP6TABLES -A INPUT -m conntrack --ctstate RELATED,ESTABLISHED -j ACCEPT

# accept ipv6 link-local traffic
$IP6TABLES -A INPUT -s fe80::/10 -j ACCEPT

# accept ipv6 multicast
$IP6TABLES -A INPUT -s ff00::/8 -j ACCEPT

# log and drop invalid flag combinations
for flags in 'ALL FIN,URG,PSH' 'ALL ALL' 'ALL SYN,RST,ACK,FIN,URG' 'ALL NONE' 'SYN,RST SYN,RST' 'SYN,FIN SYN,FIN'
do
	$IPTABLES -A INPUT -p tcp --tcp-flags ${flags} -j DROP
done

create_set allowed_udp bitmap:port range 0-65535
create_set allowed_tcp bitmap:port range 0-65535

for sfx in '' ".$(hostname -s)"
do
	if [ -e "services${sfx}" ]
	then

		for l in $(decommentcat "services${sfx}")
		do
			allow_services "${l}"
		done
	fi
done

$IPTABLES -A INPUT -i "${EXT_IF}" -p tcp -m set --match-set allowed_tcp dst -j ACCEPT
$IPTABLES -A INPUT -i "${EXT_IF}" -p udp -m set --match-set allowed_udp dst -j ACCEPT
$IP6TABLES -A INPUT -i "${EXT_IF}" -p tcp -m set --match-set allowed_tcp dst -j ACCEPT
$IP6TABLES -A INPUT -i "${EXT_IF}" -p udp -m set --match-set allowed_udp dst -j ACCEPT

# insert persistent-pest-blocker
./xenophobe.sh

# insert trusted passes
./trusted.sh

